/*
 * Copyright (c) 2008, Aaron Digulla
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of Aaron Digulla nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
package de.pdark.decentxml;

import static org.junit.Assert.*;

import org.junit.Test;

public class XMLParseExceptionTest
{
    @Test
    public void testWithNullToken () throws Exception
    {
        XMLParseException e = new XMLParseException ("x", (Token)null);
        assertEquals ("x", e.getMessage ());
        assertNull (e.getSource ());
    }
    
    @Test
    public void testWithElement () throws Exception
    {
        Element e = new Element ("xxx");
        XMLParseException ex = new XMLParseException ("test", e);
        assertEquals ("Line 1, column 1: test", ex.getMessage ());
    }
    
    @Test
    public void testOffset () throws Exception
    {
        XMLSource source = new XMLStringSource ("a\rb\r\nc\nd");
        XMLParseException e = new XMLParseException ("x", source, source.length () - 1);
        assertEquals (7, e.getOffset ());
        assertEquals ("Line 4, column 1: x", e.getMessage ());
        assertEquals (source, e.getSource ());
    }
    
    @Test
    public void testColumn () throws Exception
    {
        XMLSource source = new XMLStringSource ("a\rb\r\nc\nd");
        XMLParseException e = new XMLParseException ("x", source, source.length ());
        assertEquals (2, e.getColumn ());
        assertEquals ("Line 4, column 2: x", e.getMessage ());
    }
    
    @Test
    public void testLineStartOffset () throws Exception
    {
        XMLSource source = new XMLStringSource ("a\rb\r\nc\n");
        XMLParseException e = new XMLParseException ("x", source, source.length ());
        assertEquals (7, e.getLineStartOffset ());
        assertEquals ("Line 4, column 1: x", e.getMessage ());
    }
    
    @Test
    public void testCalcLineNumber () throws Exception
    {
        XMLSource source = new XMLStringSource ("a\rb\r\nc\r\n");
        XMLParseException e = new XMLParseException ("x", source, source.length ());
        assertEquals (1, e.getColumn ());
        assertEquals (8, e.getLineStartOffset ());
        assertEquals ("Line 4, column 1: x", e.getMessage ());
    }
    
    @Test
    public void testCalcLineNumber2 () throws Exception
    {
        XMLSource source = new XMLStringSource ("a\rb\r\nc\r");
        XMLParseException e = new XMLParseException ("x", source, source.length ());
        assertEquals (7, e.getLineStartOffset ());
        assertEquals ("Line 4, column 1: x", e.getMessage ());
    }

}
